<?php
/* Author: Michał Jałocha
   Updated by: Marcin D. "Perry" Mikielewicz
*/

$labels = [];
$labels['collapseall'] = 'Zwiń wszystkie foldery';
$labels['expandall'] = 'Rozwiń wszystkie foldery';

$messages = [];
